"""
Juice
"""

from juice import (View, flash, abort, request, url_for, redirect)
from juice.decorators import (route, menu, template, plugin, methods)
from juice.plugins import (contact_page, )

# ------------------------------------------------------------------------------

@plugin(contact_page.contact_page, menu={"name": "Contact Us", "order": 3})
class Index(View):

    @menu("Home", order=1)
    def index(self):
        self.meta_tags(title="Hello View!")
        return {}








"""
---------------------------------- JUICY -----------------------------------

A command line tool to manage your app

To run, Juice already has a hook that you can directly use in the command line

Run:

    juicy

It will show the list of actions available

To run an action:

    juicy $action


By default it will use the DEV environment. On production, change the env

    ENV=PRODUCTION juicy $action


If you are using Propel, the example below will run the setup in the production
environment

    scripts:

      # Run before all
      before_all:
        -
          command: "juicy setup"
          environment: ENV="Production"

-----

Examples:

@cli.command()
def hello():
    ''' Hello Description '''
    print(":Hello")
    print("Hello From the Other Side!")

@cli.command()
@click.option("id")
def user(id):
    ''' This is the user's description '''
    print("User ID: %s" % id)


- The hello() can be called

    juicy hello

- The user() can be called:

    juicy user id 1

--------------------------------------------------------------------------------

"""

from juice import Juice, get_env_config, abort
import juice.utils as utils
from application import config, model
from juice.plugins import user, publisher
from juice.ext import mailman
from juice.cli import cli, click

# init Juice without registering the views
Juice(__name__, skip_views=True)

# Load the config
conf = get_env_config(config)

# ------------------------------------------------------------------------------

def setup_model():
    """
    Setup models from model.py
    """
    model.db.create_all()

def setup_admin_user_publisher():
    """
    Setup Juice User and Publisher Admin
    :return:
    """

    # ==== Setup User Admin === #

    admin_email = conf.APPLICATION_ADMIN_EMAIL
    if not admin_email:
        print("ERROR: APPLICATION_ADMIN_EMAIL is empty")
        exit()

    password = utils.generate_random_string()
    if user.setup(model.User,
                  admin_email=admin_email,
                  password=password):

        if mailman.validated:
            body = "Admin Password: %s" % password
            mailman.send(to=admin_email, subject="Admin Password", body=body)

        print("---- Setup SUPER ADMIN User ----")
        print("- Admin Email: %s" % admin_email)
        print("- Admin Password: %s" % password)
        print("-" * 40)
        print("")

    # ==== Setup Publisher === #

    # admin user
    admin_user = model.User.User.get_by_email(admin_email)

    # Set default categories and post types
    post_categories = ["Uncategorized"]
    post_types = ["Page", "Blog", "Document", "Other"]

    if admin_user and admin_user.role.name.lower() == "superadmin":
        print("---- Setup PUBLISHER ----")
        if publisher.setup(model.Publisher,
                           admin_user_id=admin_user.id,
                           post_types=post_types,
                           post_categories=post_categories):
            print("Completed")
        print("-" * 40)
        print("")

# ------------------------------------------------------------------------------


@cli.command()
def setup():

    """ Setup """
    print("::Setup")
    setup_db()
    setup_admin_user_publisher()

    print("Done!")










from flask import session, make_response, request
from juice import Juice

from authomatic import Authomatic, provider_id
from authomatic.providers import oauth1, oauth2
from authomatic.adapters import WerkzeugAdapter

class OAuthomatic(object):
    oauth = None
    response = None

    def init_app(self, app):

        if app.config.get("MODULE_USER_ACCOUNT_ENABLE_OAUTH_LOGIN"):
            secret = app.config.get("SECRET_KEY")
            providers = app.config.get("MODULE_USER_ACCOUNT_OAUTH_PROVIDERS")
            config = {}
            auth_providers = []

            for provider, conf in providers.items():
                if hasattr(oauth2, provider):
                    cls = getattr(oauth2, provider)
                    conf["class_"] = conf["class_"] if "class_" in conf else cls
                elif hasattr(oauth1, provider):
                    cls = getattr(oauth1, provider)
                    conf["class_"] = conf["class_"] if "class_" in conf else cls
                else:
                    continue

                conf["id"] = provider_id()
                _provider = provider.lower()
                auth_providers.append(_provider)
                config[_provider] = conf

            self.oauth = Authomatic(
                config=config,
                secret=secret,
                session=session,
                report_errors=True
            )

            Juice.g(OAUTH_PROVIDERS=auth_providers)

    def login(self, provider):
        response = make_response()
        adapter = WerkzeugAdapter(request, response)
        login = self.oauth.login(adapter=adapter,
                                 provider_name=provider,
                                 session=session,
                                 session_saver=self._session_saver)
        self.response = response
        return login

    def _session_saver(self):
        session.modified = True

#oauth = OAuthomatic()
#Juice.bind(oauth.init_app)



"""
A utils for Markdown

html : render markdown to html
toc : Get the Table of Content
extract_images: Return a list of images, can be used to extract the top image

"""

import markdown
from markdown.treeprocessors import Treeprocessor
from markdown.extensions import Extension

###
# This extension will extract all the images from the doc
class ExtractImagesExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        ext = ExtractImagesTreeprocessor(md)
        md.treeprocessors.add("imageextractor", ext, "_end")

class ExtractImagesTreeprocessor(Treeprocessor):
    def run(self, root):
        "Find all images and append to markdown.images. "
        self.markdown.images = []
        for image in root.getiterator("img"):
            self.markdown.images.append(image.attrib["src"])

###

# LazyImageExtension
# An extension to delay load of images on the page
class LazyImageExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        ext = LazyImageTreeprocessor(md)
        md.treeprocessors.add("lazyimage", ext, "_end")

class LazyImageTreeprocessor(Treeprocessor):
    def run(self, root):
        for image in root.getiterator("img"):
            image.set("data-src", image.attrib["src"])
            image.set("src", "")
            image.set("class", "lazy")


# EMBED
# [[embed]](http://)
# An extension to delay load of images on the page.
# It adds the class oembed in the link
class OEmbedExtension(Extension):
    def extendMarkdown(self, md, md_globals):
        ext = OEmbedTreeprocessor(md)
        md.treeprocessors.add("oembedextension", ext, "_end")

class OEmbedTreeprocessor(Treeprocessor):
    def run(self, root):
        for a in root.getiterator("a"):
            if a.text.strip() == "[embed]":
                a.text = ""
                a.set("class", "oembed")
                a.set("target", "_blank")

# ------------------------------------------------------------------------------
def html(text, lazy_images=False):
    """
    To render a markdown format text into HTML.

    - If you want to also build a Table of Content inside of the markdow,
    add the tags: [TOC]
    It will include a <ul><li>...</ul> of all <h*>

    :param text:
    :param lazy_images: bool - If true, it will activate the LazyImageExtension
    :return:
    """
    extensions = [
        'markdown.extensions.nl2br',
        'markdown.extensions.sane_lists',
        'markdown.extensions.toc',
        'markdown.extensions.tables',
        OEmbedExtension()
    ]
    if lazy_images:
        extensions.append(LazyImageExtension())

    return markdown.markdown(text, extensions=extensions)

def toc(text):
    """
    Return a table of context list
    :param text:
    :return:
    """
    extensions = ['markdown.extensions.toc']
    mkd = markdown.Markdown(extensions=extensions)
    html = mkd.convert(text)
    return mkd.toc

def extract_images(text):
    """
    Extract all images in the content
    :param text:
    :return:
    """
    extensions = [ExtractImagesExtension()]
    mkd = markdown.Markdown(extensions=extensions)
    html = mkd.convert(text)
    return mkd.images

# ------------------------------------------------------------------------------




from juice import (View, register_package, abort)

register_package(__package__)

def view(template=None):
    """
    Create the Maintenance view
    Must be instantiated

    import maintenance_view
    MaintenanceView = maintenance_view()

    :param template_: The directory containing the view pages
    :return:
    """
    if not template:
        template = "Juice/Plugin/MaintenancePage/index.html"

    class Maintenance(View):

        @classmethod
        def register(cls, app, **kwargs):
            super(cls, cls).register(app, **kwargs)

            if cls.get_config("APPLICATION_MAINTENANCE_ON"):
                app.logger.info("APPLICATION MAINTENANCE PAGE IS ON")

                @app.before_request
                def on_maintenance():
                    return cls.render_(layout_=template), 503

    return Maintenance

Maintenance = view()



__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "Juice"
__version__ = "0.0.13"
__summary__ = "A full stack Flask based framework that put structure in the " \
              "file system. Features: Caching, Mailing, Storage, CSRF, recaptcha " \
              "and more"
__uri__ = "https://github.com/mardix/juice"
__author__ = "Mardix"
__email__ = "mardix@pylot.io"
__license__ = "MIT"
__copyright__ = "(c) 2016 %s" % __author__




"""
::Juice::

https://github.com/mardix/juice

{project_name}.py

This is the entry point of the application.

--------------------------------------------------------------------------------

** To serve the local development app

> juicy serve {project_name}

#---------

** To deploy with Propel ( https://github.com/mardix/propel )

> propel -w

#---------

** To deploy with Gunicorn

> gunicorn {project_name}:app

"""

from juice import Juice

# Import the project's views
import application.{project_name}.views

# 'app' variable name is required if you intend to use 'juicy' the cli tool
app = Juice(__name__, project="{project_name}")




"""
Juice: API
"""

from juice import (View, abort, request)
from juice.decorators import (route, methods, render_as_json, render_as_xml)
from juice.plugins import error_page


@render_as_json
def error_renderer(error):
    """
    Custom render output for API endpoint
    When abort() is invoked, it will render this function
    :param error: The error sent
    :return: dict
    """
    return {
        "error": True,
        "description": error.description,
        "code": error.code
    }, error.code
error_page.renderer = error_renderer


@render_as_json
class Index(View):
    """ API Endpoints """

    def index(self):
        return {
            "name": self.get_config("APPLICATION_NAME"),
            "version": self.get_config("APPLICATION_VERSION")
        }

    def get(self, id):
        return {
            "description": "This is a get",
            "id": id
        }

    def test(self, id):
        return {
            "desc": "This is a test",
            "id": id
        }

    def error(self):
        abort(400, "Custom Message")


